from pathlib import Path
from typing import NamedTuple

from colour import Color


class Settings(NamedTuple):
    """Tools settings"""

    in_file: Path
    """Input file"""

    out_file: Path
    """Output file"""

    thickness: int
    """Thickness of the line, used in algorithm to set blur radius"""

    alpha: float
    """Threshold for background from zero to one.
    The closer to one, the more pixels will be left"""

    color: Color
    """Final color of contour"""

    crop: bool
    """Crop transparent pixels after converting"""



import os
from pathlib import Path

import click
from colour import Color
from loguru import logger
from PIL import Image

from .classes import Settings
from .clean import clean_image


@click.command()
@click.option("-i", "--input", type=Path, required=True, help="Input filepath")
@click.option("-o", "--output", type=Path, required=True, help="Output filepath")
@click.option(
    "-t",
    "--thickness",
    type=int,
    help="Thickness of the line",
    default=3,
    show_default=True,
)
@click.option(
    "-a",
    "--alpha",
    type=float,
    help="From 0 to 1: the closer to one, the more pixels will be left",
    default=0.3,
    show_default=True,
)
@click.option(
    "-c",
    "--color",
    type=str,
    help="Color of result contour",
    default="black",
    show_default=True,
)
@click.option(
    "-p",
    "--crop",
    type=bool,
    help="Crop transparent pixels after converting",
    default=True,
    show_default=True,
)
def process(
    input: Path, output: Path, thickness: int, alpha: float, color: Color, crop: bool
) -> None:
    """Take contour from selected file"""

    parsed_color = Color(color)

    settings = Settings(
        in_file=input,
        out_file=output,
        thickness=thickness,
        alpha=alpha,
        color=parsed_color,
        crop=crop,
    )

    initial_image = Image.open(input)
    result_image = clean_image(initial_image, settings)
    if os.path.exists(output):
        os.remove(output)
    result_image.save(output)

    logger.info(f"{input} --> {output}")


if __name__ == "__main__":
    process()



import cv2
import numpy as np

from .classes import Settings


def find_mask(img: np.ndarray, settings: Settings) -> np.ndarray:
    """Find contour and return mask image with black background and white contour"""
    blured = cv2.blur(img, (3, 3))
    edges = cv2.Canny(blured, settings.lower, settings.upper)

    return edges



from typing import Tuple

import numpy as np
from colour import Color

from monochromist.opencv_version.classes import Settings


def color_image(img: np.ndarray, settings: Settings) -> np.ndarray:
    """Color non-transparent pixels with selected color

    :param img: input image with black (0) background and white contour (255)
    :param settings: user settings

    :return: numpy array with transparent background and colored contour"""

    color_tuple = color2tuple(settings.color)
    transparent = (0, 0, 0, 0)

    colored = np.array([color_tuple if x else transparent for x in img.flatten()])
    height, width = img.shape

    return colored.reshape((height, width, -1))


def crop_borders(img: np.ndarray) -> np.ndarray:
    notna_columns = img.any(axis=0)
    notna_rows = img.any(axis=1)

    left = np.flatnonzero(notna_columns)[0]
    right = np.flatnonzero(notna_columns)[-1]

    upper = np.flatnonzero(notna_rows)[0]
    lower = np.flatnonzero(notna_rows)[-1]

    return img[upper:lower, left:right]


def color2tuple(color: Color) -> Tuple[int, int, int, int]:
    """Convert color to RGBA tuple"""
    r, g, b = [int(255 * x) for x in color.rgb]
    alpha_channel = 255
    # opencv use BGR not RGB model
    return b, g, r, alpha_channel



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['monochromist', 'monochromist.math_version', 'monochromist.opencv_version']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=8.2.0,<9.0.0',
 'click>=7.1.2,<8.0.0',
 'colour>=0.1.5,<0.2.0',
 'loguru>=0.5.3,<0.6.0',
 'numpy>=1.20.2,<2.0.0',
 'opencv-python>=4.5.1,<5.0.0']

entry_points = \
{'console_scripts': ['monochromist = '
                     'monochromist.opencv_version.process:process',
                     'monochromist_v2 = '
                     'monochromist.math_version.process:process']}

setup_kwargs = {
    'name': 'monochromist',
    'version': '0.1.1',
    'description': 'Convert your sketch into contour with transparent background',
    'long_description': '# monochromist\n\nFrom sketch to contour with transparent background:\n\n![example](doc/images/cat_illustration.png)\n\nMinimum example:\n```bash\nmonochromist -i <input image> -o <output image> -c "<color>"\n```\n\nFor example:\n```bash\nmonochromist -i doc/images/cat.jpg -o /tmp/cat.png -c "black"\n```\n\nFor more settings run\n```bash\nmonochromist --help\n```',
    'author': 'Bogdan Neterebskii',
    'author_email': 'bog2dan1@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ozzzzz/monochromist',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Tuple

import numpy as np
from colour import Color
from PIL import Image, ImageFilter

from .classes import Settings


def clean_image(img: Image, settings: Settings) -> Image:
    """Clean background and color contour to selected color"""

    # Convert to grayscale and blur picture.
    converted = img.convert("L")

    # Blur filter is used to average background
    blured = converted.filter(ImageFilter.MedianFilter(settings.thickness))
    values = np.asarray(blured)

    threshold = find_threshold(values, settings)
    cleaned_array = values < threshold
    colored_image = color_image(img, cleaned_array, settings)

    # TODO: remove artifacts with alone pixels

    if settings.crop and cleaned_array.any():
        borders = find_borders(cleaned_array)
        colored_image = colored_image.crop(borders)

    return colored_image


def find_threshold(arr: np.ndarray, settings: Settings) -> float:
    """Find threshold using percentiles.
    Main idea: pixels from contour happens very rarely and should be near 0-th percentile"""

    flattened = arr.flatten()
    step_for_percentiles = 10
    percentiles = [np.percentile(flattened, i, axis=0) for i in range(0, 101, step_for_percentiles)]
    return (1 - settings.alpha) * percentiles[0] + settings.alpha * np.mean(percentiles[1:])


def color_image(img: Image, arr: np.array, settings: Settings) -> Image:
    """Color non-transparent pixels with selected color"""

    color_tuple = color2tuple(settings.color)
    transparent = (0, 0, 0, 0)

    colored = [color_tuple if x else transparent for x in arr.flatten()]
    new_image = img.convert("RGBA")
    new_image.putdata(colored)
    return new_image


def find_borders(arr: np.array) -> Tuple[int, int, int, int]:
    """Find transparent borders"""

    notna_columns = arr.any(axis=0)
    notna_rows = arr.any(axis=1)

    left = np.flatnonzero(notna_columns)[0]
    right = np.flatnonzero(notna_columns)[-1]

    upper = np.flatnonzero(notna_rows)[0]
    lower = np.flatnonzero(notna_rows)[-1]

    return left, upper, right, lower


def color2tuple(color: Color) -> Tuple[int, int, int, int]:
    """Convert color to RGBA tuple"""
    r, g, b = [int(255 * x) for x in color.rgb]
    alpha_channel = 255
    return r, g, b, alpha_channel



import os
from pathlib import Path

import click
import cv2
from colour import Color
from loguru import logger

from monochromist.opencv_version.post_editing import color_image, crop_borders

from .classes import Settings
from .find_mask import find_mask


@click.command()
@click.option("-i", "--input", type=Path, required=True, help="Input filepath")
@click.option("-o", "--output", type=Path, required=True, help="Output filepath")
@click.option(
    "-l",
    "--lower",
    type=int,
    help="Lower threshold for Candy Edge detection",
    default=100,
    show_default=True,
)
@click.option(
    "-u",
    "--upper",
    type=int,
    help="Lower threshold for Candy Edge detection",
    default=200,
    show_default=True,
)
@click.option(
    "-c",
    "--color",
    type=str,
    help="Color of result contour",
    default="black",
    show_default=True,
)
@click.option(
    "-p",
    "--crop",
    type=bool,
    help="Crop transparent pixels after converting",
    default=True,
    show_default=True,
)
def process(input: Path, output: Path, lower: int, upper: float, color: Color, crop: bool) -> None:
    """Take contour from selected file"""

    parsed_color = Color(color)

    settings = Settings(
        in_file=input,
        out_file=output,
        lower=lower,
        upper=upper,
        color=parsed_color,
        crop=crop,
    )

    initial_image = cv2.imread(str(input), cv2.IMREAD_GRAYSCALE)

    mask = find_mask(initial_image, settings)

    cropped = mask
    if settings.crop:
        cropped = crop_borders(mask)

    colored = color_image(cropped, settings)

    if os.path.exists(output):
        os.remove(output)
    cv2.imwrite(str(output), colored)

    logger.info(f"{input} --> {output}")


if __name__ == "__main__":
    process()
